/*
 * lesson2.c for VIC-20 using cc65.
**
** Mike 'Hawk' Hawkey (mike.hawkey at internode.on.net)
**
*/

#define __VIC20__
#define DEBUG
#define EXPANDED_MEM

#include "vic20.h"
#include <conio.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "menu_data.h"

/******************************************************************************
**                  T y p e   D e f i n i t i o n s
******************************************************************************/
typedef enum
{
  false,
  true
} boolean;

typedef struct
{
  char    xLoc;
  char    yLoc;
  char    microStep;
  char    direction;
  char    state;
  char    *characters[6];
} tPlayer;

/******************************************************************************
**                  C o n s t a n t s
******************************************************************************/
#define LIMIT_OF_MEMORY   (*(unsigned char*)0x37)

#define BANK_SIZE         (0x2000)
#define BANK_1            (*(unsigned char)0x2000)
#define BANK_2            (*(unsigned char)0x4000)
#define BANK_3            (*(unsigned char)0x6000)
#define BANK_5            (*(unsigned char)0xA000)

//#define SCREEN_WIDTH      (20)
//#define SCREEN_HEIGHT     (12)
//#define SCREEN_SIZE       (240) /*(SCREEN_WIDTH * SCREEN_HEIGHT)*/
#define SCREEN_WIDTH      (22)
#define SCREEN_HEIGHT     (23)
#define SCREEN_SIZE       (506) /*(SCREEN_WIDTH * SCREEN_HEIGHT)*/
#ifdef NTSC
#define SCREEN_X_OFFSET   (2)
#define SCREEN_Y_OFFSET   (8)
#elif PAL
#define SCREEN_X_OFFSET   (14)
#define SCREEN_Y_OFFSET   (36)
#else
#error "Video standard not defined. PAL or NTSC?"
#endif

#define DEFAULT_VOLUME    (0xF)

#ifdef PAL
  #define LINES           (312)
  #define CYCLES_PER_LINE (71)
#else
  #define LINES           (261)
  #define CYCLES_PER_LINE (65)
#endif
#define TIMER_VALUE       (LINES * CYCLES_PER_LINE - 2)

#define DOUBLE_HEIGHT     (0x01)

//#define waitvsync()   while((signed char)VIC.rasterline >= 0)
#define waitvsync(x)  while((signed char)VIC.rasterline != (char)x)

const char* uppercase = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
const char* lowercase = "abcdefghijklmnopqrstuvwxyz";
static const char Text [] = "Hello world!";

/******************************************************************************
**                  D a t a
******************************************************************************/
extern char*	image_list[];
unsigned char* screenAddr;
unsigned char* colourAddr;
unsigned char* charmapAddr;
unsigned char* screen_y[SCREEN_HEIGHT];
unsigned char* colour_y[SCREEN_HEIGHT];


/******************************************************************************
**                  P r o t o t y p e s
******************************************************************************/
void __fastcall__ CopySprite(char* dest, char* src, char offset);
void __fastcall__ ShiftSprite(char* sprite, char direction);
void __fastcall__ InitHiResGraphics(void);


/******************************************************************************
**                  C o d e
******************************************************************************/

void IntroScreen(void)
{
}

/*
void Print(int left, int top, char *str)
{
    int pos = 0;
    while (*str)
    {
        DrawChar(left + pos, top, *str++);
        pos += 8;
    }
}
*/

void __fastcall__ CopySprite(char* dest, char* src, char offset)
{
  unsigned char y;
  char* dest2;
  char  offset2;
  offset2=8-offset;
  dest2 = dest + 24;
  /*
  **  Copy blank lines before data
  */
  for(y=0; y<offset; ++y)
  {
    *dest=0;
    *dest2=0;
    ++dest;
    ++dest2;
  }
  src+=2;                 // Increment past the size bytes
  /*
  **  Copy data
  */
  for(y=0; y<16; ++y)
  {
    *dest  = *src;
    ++dest;
    ++src;
    *dest2 = *src;
    ++dest2;
    ++src;
  }
  /*
  **  Copy blank lines after data
  */
  for(y=0; y<offset2; ++y)
  {
    *dest=0;
    *dest2=0;
    ++dest;
    ++dest2;
  }
}

void __fastcall__ InitHiResGraphics(void)
{
  unsigned char i;
  unsigned char x;
  unsigned char y;

  cputs("[TRC]InitHiResGraphics:Enter\r\n");
  cgetc();
  /*
  ** Set the Screen Size and offset and fill with
  ** spaces.
  */
  VIC.charsperline = (VIC.charsperline & 0x80) | SCREEN_WIDTH;
  VIC.linecount = (unsigned char)(SCREEN_HEIGHT<<1); // | DOUBLE_HEIGHT;
//  VIC.leftborder = (unsigned char)SCREEN_X_OFFSET;
//  VIC.upperborder = (unsigned char)SCREEN_Y_OFFSET;
  VIC.volume_color = (COLOR_WHITE<<4) | DEFAULT_VOLUME;
  VIC.bg_border_color = (COLOR_BLACK<<4) | COLOR_CYAN | COMMON_BACKGROUND;
  VIC.addr = ((VIC.addr & 0x0F) | 0xC0);
  VIC.bg_border_color = 0x08;
  screenAddr = (char *)(((VIC.addr & 0x70) | ((VIC.charsperline & 0x80)>>4))<<6);
  colourAddr = (char *)(0x9400 + ((VIC.charsperline & 0x80)<<2));
  charmapAddr = (char *)(((VIC.addr & 0x07)<<10) + ((VIC.addr & 0x08) ? 0x8000 : 0));
#ifdef DEBUG
//  clrscr();
//  cprintf("screenAddr=%04X\r\n", screenAddr);
//  cprintf("colourAddr=%04X\r\n", colourAddr);
//  cprintf("charmapAddr=%04X\r\n", charmapAddr);
//  cgetc();
#endif
  /*
  ** Initialise screen_y and colour_y
  */
  for (i=0; i<SCREEN_HEIGHT; ++i)
  {
    screen_y[i] = screenAddr + (i * SCREEN_WIDTH);
    colour_y[i] = colourAddr + (i * SCREEN_WIDTH);
  }
#ifdef DEBUG
  /*
  ** ------------------------------------------------------------------
  **   D e b u g   C o d e
  ** ------------------------------------------------------------------
  */
  /*
  ** Display full character set on the screen
  */
  x=0;
  y=1;
  for (i=0; y<11; ++i)
  {
    *(char *)(screen_y[y]+x)=i;
    ++x;
    if (x==16)
    {
      x=0;
      y++;
    }
  }
  cgetc();

  /*
  ** Display 2nd copy of character set on the screen in multicolour mode
  */

  x=0;
  y=1;
  for (i=0; y<11; ++i)
  {
    *(char *)(screen_y[y]+x) = i;
    *(char *)(colour_y[y]+x) = MULTICOLOR | COLOR_RED;
    ++x;
    if (x==16)
    {
      x=0;
      y++;
    }
  }
  cgetc();
#endif
  /*
  ** ------------------------------------------------------------------
  */
    
  /* Clear the screen
   */
  for (i=1; i<SCREEN_HEIGHT-1; ++i)
  {
    memset(screen_y[i], 0, SCREEN_WIDTH);
  }
  /*
  ** Copy character set from ROM, and switch to RAM Character Set
  */
  VIC.addr = (VIC.addr & 0xF0) | 0x0D;    // RAM Address 0x1000
  charmapAddr = (char *)(0x1400);
  memcpy(charmapAddr, &CharacterGraphics, 0x0800);
  cgetc();

  memcpy(screen_y[0], &menu_ci, SCREEN_WIDTH);
  cgetc();
  memcpy(colour_y[0], &menu_at, SCREEN_WIDTH);
  cgetc();
  memset(screen_y[1], 0, 220);
  cgetc();

  memset(colour_y[1], COLOR_BLUE, SCREEN_WIDTH*(SCREEN_HEIGHT-1));
  cgetc();
  memcpy(screen_y[SCREEN_HEIGHT-1], &menu_ci_last_line, SCREEN_WIDTH);
  cgetc();
  memcpy(colour_y[SCREEN_HEIGHT-1], &menu_at_last_line, SCREEN_WIDTH);
  cgetc();

#ifdef DEBUG
  /*
  ** Display full character set on the screen
  */
  x=0;
  y=1;
  for (i=0; y<11; ++i)
  {
    *(char *)(screen_y[y]+x)=i;
    ++x;
    if (x==16)
    {
      x=0;
      y++;
    }
  }
  cgetc();
    
  /*
  ** Copy character set from ROM, and switch to RAM Character Set
  */
  VIC.addr = (VIC.addr & 0xF0) | 0x0D;    // RAM Address 0x1000
  charmapAddr = (char *)(0x1400);
  memcpy(charmapAddr, &CharacterGraphics, 0x0800);

#endif
  cputs("[TRC]InitHiResGraphics:Enter\r\n");
}

void main (void)
{
  char i;
  unsigned char XSize, YSize;

  clrscr();
  screensize(&XSize, &YSize);
  cputs("lowercase\r\n");
  cputs("abcdefghijklmnopqrstuvwxyz0123456789\r\n");
  cputs("UPPERCASE\r\n");
  cputs("ABCDEFGHIJKLMNOPQRSTUVWXYZ!@#$%^&*()_+\r\n");
  cgetc();
//  cprintf("%dx%d\r\n",XSize,YSize);
  /* Write the greeting in the mid of the screen */
//  gotoxy ((XSize - strlen (Text)) / 2, YSize / 2);
//  cprintf ("%s", Text);
//  cgetc();

  InitHiResGraphics(); 
  cgetc();
  for (i=1; i<10; i++)
  {
    gotoxy(0,i); 
    cputs(image_list[i-1]);
      memcpy(screen_y[i], image_list[i-1], strlen(image_list[i-1]));
  } 
  cgetc();
  memcpy(screen_y[1], uppercase, SCREEN_WIDTH);
  cgetc();
  memcpy(screen_y[2], lowercase, SCREEN_WIDTH);
  cgetc();
  clrscr();
  screensize(&XSize, &YSize);
  cputs("1\n");
  cgetc();
  cputs("2\n");
  cgetc();
  cputs("3\r");
  cgetc();
  cputs("4\r");
  cgetc();
  cputs("5\r\n");
  cgetc();
  clrscr();
  cputs("abcdefghijklmnopqrstuvwxyz0123456789");
  cgetc();
  cputs("ABCDEFGHIJKLMNOPQRSTUVWXYZ!@#$%^&*()_+");
  cgetc();
  /* Write the greeting in the mid of the screen */
  cprintf("%dx%d",XSize,YSize);
  gotoxy ((XSize - strlen (Text)) / 2, YSize / 2);
  cprintf ("%s", Text);
  while(1);
}



